#include <FastLED.h>

#define LED_PIN  A9 //a define is just a substitution. where you see LED_PIN it inserts A9. Unlike an varible it can't be changed
#define MAX_BRIGHTNESS 25
#define NUM_LEDS 64

#define trail_length 8


byte up_pin = 10;
byte left_pin = 7;
byte down_pin = 8;
byte right_pin = 9;

byte action_pin = 11;

byte p_up, p_down, p_left, p_right, p_act;
byte c_up, c_down, c_left, c_right, c_act;
byte pos_change;
byte hue_shift;

int interval1 = 40;
byte hue;
int hue_latch;
unsigned long current_millis;         // will store the current "time"
unsigned long  previous_millis1 = 0;  // will store last time LED1 was updated
int led_count;
int distance = 127;
int pos_temp;
int xpos = 4;
int ypos = 4;

byte trail[trail_length];

CRGB led_grid[NUM_LEDS];

void setup() {
  FastLED.addLeds<WS2812B, LED_PIN, GRB>(led_grid, NUM_LEDS).setCorrection(TypicalSMD5050);
  FastLED.setBrightness( MAX_BRIGHTNESS );  //0 (0FF) - 255 (ABSURDLY BRIGHT)

  randomSeed(A8); //this makes the random() expresion closer to actually random. Otherwise it will go through the same sequence every time.
  //set it to an anlog pin with nothing attached

  pinMode(up_pin, INPUT_PULLUP);
  pinMode(down_pin, INPUT_PULLUP);
  pinMode(left_pin, INPUT_PULLUP);
  pinMode(right_pin, INPUT_PULLUP);
  pinMode(action_pin, INPUT_PULLUP);

  pos_temp = xpos + (ypos * 8);

  for (int i = 0; i < trail_length ; i++) {
    trail[i] = pos_temp;  // set the whole array to the current position
  }
}


void loop()
{
  current_millis = millis();

  p_up = c_up;
  c_up = digitalRead(up_pin);
  if (c_up == 0 && p_up == 1) {
    ypos -= 1;
    pos_change = 1;
    if (ypos < 0) {
      ypos = 0;
    }
  }

  p_down = c_down;
  c_down = digitalRead(down_pin);
  if (c_down == 0 && p_down == 1) {
    ypos += 1;
    pos_change = 1;

    if (ypos > 7) {
      ypos = 7;
    }
  }

  p_left = c_left;
  c_left = digitalRead(left_pin);
  if (c_left == 0 && p_left == 1) {
    xpos -= 1;
    pos_change = 1;
    if (xpos < 0) {
      xpos = 0;
    }
  }

  p_right = c_right;
  c_right = digitalRead(right_pin);
  if (c_right == 0 && p_right == 1) {
    xpos += 1;
    pos_change = 1;
    if (xpos > 7) {
      xpos = 7;
    }
  }


  if (pos_change == 1) {
    pos_change = 0;
    pos_temp = xpos + (ypos * 8);
    for (int i = trail_length - 1; i > 0 ; i--) {
      trail[i] = trail[i - 1];
    }
    trail[0] = pos_temp;
  }

  if (c_down == 0 && c_up == 0) { // if these two buttons are down
    pos_temp = xpos + (ypos * 8);

    for (int i = 0; i < trail_length ; i++) {
      trail[i] = pos_temp;  // set the whole array to the current position
    }
  }



  if (current_millis - previous_millis1 > interval1) {
    previous_millis1 = current_millis;

    hue -= 4;

    Serial.println(hue);

    for (int i; i < NUM_LEDS; i++) {
      led_grid[i]  = CHSV( 0, 0, 0);
    }
    hue_shift = 0;
    for (int i; i < trail_length - 1; i++) {
      hue_shift += 8;
      led_grid[trail[i]]  = CHSV( hue + hue_shift, 255, 255); //turn one light at our position on
      Serial.print(trail[i]);
      Serial.print(" ");
    }
    led_grid[trail[0]]  = CHSV( hue + hue_shift, 32, 255); //turn one light at our position on

    Serial.println(" ");

    FastLED.setBrightness(MAX_BRIGHTNESS);  // change the brightness of everything
    FastLED.show();  // send the array data to the leds

  }


}





