#include <Audio.h>
#include <Wire.h>
#include <SPI.h>
#include <SD.h>
#include <SerialFlash.h>

// GUItool: begin automatically generated code
AudioSynthNoisePink      pink1;          //xy=102,429
AudioSynthSimpleDrum     drum2;          //xy=190,332
AudioSynthSimpleDrum     drum1;          //xy=197,285
AudioSynthSimpleDrum     drum3;          //xy=201,378
AudioEffectEnvelope      envelope1;      //xy=261,428
AudioMixer4              mixer1;         //xy=457,351
AudioOutputAnalog        dac1;           //xy=652,346
AudioConnection          patchCord1(pink1, envelope1);
AudioConnection          patchCord2(drum2, 0, mixer1, 1);
AudioConnection          patchCord3(drum1, 0, mixer1, 0);
AudioConnection          patchCord4(drum3, 0, mixer1, 2);
AudioConnection          patchCord5(envelope1, 0, mixer1, 3);
AudioConnection          patchCord6(mixer1, dac1);
// GUItool: end automatically generated code

#include <FastLED.h>

#define LED_PIN  A9 //a define is just a substitution. where you see LED_PIN it inserts A9. Unlike an varible it can't be changed
#define MAX_BRIGHTNESS 25
#define NUM_LEDS 64

CRGB led_grid[NUM_LEDS];

#define seq_length 8
#define down_pin 8


unsigned long current_millis;
unsigned long  previous_millis[3];

int   rate1 = 145;
int seq1_count;
int rate2=1;
int pot1_value;
int pot2_value;
float freq1;
float freq2;
float level_outl;
int r1;
int drum3_latch;
int drum3_length;
float drum2_freq;
int drum3_start_time;
int button_state;
int drum_bright[4];
int drum1_seq[seq_length] = {1, 0, 0, 0,  1, 0, 0, 1};
int drum2_seq[seq_length] = {0, 0, 1, 1,  0, 0, 0, 0};
int drum3_seq[seq_length] = {0, 1, 0, 1,  0, 1, 0, 1};

byte drum_frame1 [64] =
{ 1, 1, 1, 1, 1, 1, 1, 1,
  1, 0, 0, 0, 0, 0, 0, 1,
  1, 0, 0, 0, 0, 0, 0, 1,
  1, 0, 0, 0, 0, 0, 0, 1,
  1, 0, 0, 0, 0, 0, 0, 1,
  1, 0, 0, 0, 0, 0, 0, 1,
  1, 0, 0, 0, 0, 0, 0, 1,
  1, 1, 1, 1, 1, 1, 1, 1
};

byte drum_frame2 [64] =
{ 0, 0, 0, 0, 0, 0, 0, 0,
  0, 1, 1, 1, 1, 1, 1, 0,
  0, 1, 0, 0, 0, 0, 1, 0,
  0, 1, 0, 0, 0, 0, 1, 0,
  0, 1, 0, 0, 0, 0, 1, 0,
  0, 1, 0, 0, 0, 0, 1, 0,
  0, 1, 1, 1, 1, 1, 1, 0,
  0, 0, 0, 0, 0, 0, 0, 0,
};

byte drum_frame3 [64] =
{ 0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 1, 1, 1, 1, 0, 0,
  0, 0, 1, 0, 0, 1, 0, 0,
  0, 0, 1, 0, 0, 1, 0, 0,
  0, 0, 1, 1, 1, 1, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0,
  0, 0, 0, 0, 0, 0, 0, 0,
};


void setup() {

  AudioMemory(200); //255 is max
  FastLED.addLeds<WS2812B, LED_PIN, GRB>(led_grid, NUM_LEDS).setCorrection(TypicalSMD5050);

  pinMode(down_pin, INPUT_PULLUP);


  mixer1.gain(0, .5);
  mixer1.gain(1, .5);
  mixer1.gain(2, .5);
  mixer1.gain(3, .2);

  pink1.amplitude(1);

}

void loop()
{

  current_millis = millis();

  button_state = digitalRead(down_pin);


  if (current_millis - previous_millis[0] > rate1 && button_state == 0) {
    previous_millis[0] = current_millis;

    r1 = random(20); //get a new random number between 0 and 20;

    seq1_count++;
    if (seq1_count >= seq_length) {
      seq1_count = 0;
    }

    if (drum1_seq[seq1_count] == 1) {
      drum1.frequency(160);
      drum1.length(50);
      drum1.pitchMod(.4); //.5 is no change, less is pich up, more is pitch down
      drum1.noteOn();
      drum_bright[1]=255;

    }


    if (drum2_seq[seq1_count] == 1) {
      drum2_freq = 300.00 * ((r1 / 20.00) + 1.00);
      drum2.frequency(drum2_freq);
      drum2.length(80 + r1);
      drum2.pitchMod(.6 + (r1 * .01));
      drum2.noteOn();
      drum_bright[2]=255;

    }



    if (drum3_seq[seq1_count] == 1) {
      drum3_length = 10;
      envelope1.release(15); // how long will it take for the sound to fade out after "noteOff"
      envelope1.decay(5); //not long after "noteOn" untill the level definind in sustain
      envelope1.sustain(.5); // amplitude the sound drops to after the "decay"
      envelope1.noteOn(); //start the envelop
      drum3_latch = 1;
      drum3_start_time = current_millis;
      drum_bright[3]=255;
    }




    for (int i; i < NUM_LEDS; i++) {
      if (drum_frame1[i] == 1) {
        led_grid[i]  = CHSV( random(130, 140), 255, drum_bright[1]);
      }
      if (drum_frame2[i] == 1) {
        led_grid[i]  = CHSV( random(150, 165), 255, drum_bright[2]);
      }
      if (drum_frame3[i] == 1) {
        led_grid[i]  = CHSV( random(200, 220), random(0, 20), drum_bright[3]);
      }
    }



    FastLED.setBrightness(MAX_BRIGHTNESS);  // change the brightness of everything
    FastLED.show();  // send the array data to the leds

    Serial.print(" proc:");
    Serial.print(AudioProcessorUsageMax());
    Serial.print("  mem:");
    Serial.println(AudioMemoryUsageMax());
    Serial.print(" seq1_count:");
    Serial.println(seq1_count);
    Serial.println();


    AudioProcessorUsageMaxReset();
    AudioMemoryUsageMaxReset();

  } // end of rate1 timer


  if (drum3_latch == 1) {
    if (current_millis - drum3_start_time > drum3_length) {
      envelope1.noteOff();
      drum3_latch = 0;
    }
  }


  if (current_millis - previous_millis[1] > rate2 ) {
    previous_millis[1] = current_millis;
    for (int i = 1; i < 4 ; i++) {
      drum_bright[i] -=2;
      if (drum_bright[i]<=10){
        drum_bright[i]=0;
      }
    }
  }



} //end of loop

